==> courier-imap_0.31.orig.tar.gz: courier-imap-0.31/imap/storeinfo.c <==
/*
** Copyright 1998 - 1999 Double Precision, Inc.
** See COPYING for distribution information.
*/

#if	HAVE_CONFIG_H
#include	"config.h"
#endif
#if     HAVE_UNISTD_H
#include        <unistd.h>
#endif
#include	"imaptoken.h"
#include	"imapscanclient.h"
#include	"imapwrite.h"
#include	"storeinfo.h"
#include	"maildir/maildirquota.h"
#include	"maildir/maildirmisc.h"

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#if	HAVE_UTIME_H
#include	<utime.h>
#endif
#if TIME_WITH_SYS_TIME
#include	<sys/time.h>
#include	<time.h>
#else
#if HAVE_SYS_TIME_H
#include	<sys/time.h>
#else
#include	<time.h>
#endif
#endif

static const char rcsid[]="$Id: storeinfo.c,v 1.4 2000/02/14 05:11:37 mrsam Exp $";

extern int is_trash(const char *);
extern int get_flagname(const char *, struct imapflags *);
extern int get_flags(struct imapflags *);
extern void get_message_flags( struct imapscanmessageinfo *,
	char *, struct imapflags *);
extern int reflag_filename(struct imapscanmessageinfo *, struct imapflags *,
	int);
extern void fetchflags(unsigned long);
extern void maildir_mkfilename(const char *, struct imapflags *,
	unsigned long, char **, char **);

extern struct imapscaninfo current_mailbox_info;
extern char *current_mailbox;

int storeinfo_init(struct storeinfo *si)
{
struct imaptoken *t=currenttoken();
const char *p;

	if (t->tokentype != IT_ATOM)	return (-1);
	si->plusminus=0;
	si->silent=0;

	p=t->tokenbuf;
	if (*p == '+' || *p == '-')
		si->plusminus= *p++;
	if (strncmp(p, "FLAGS", 5))	return (-1);
	p += 5;
	if (*p)
	{
		if (strcmp(p, ".SILENT"))	return (-1);
		si->silent=1;
	}

	memset(&si->flags, 0, sizeof(si->flags));

	t=nexttoken();
	if (t->tokentype == IT_LPAREN)
	{
		if (get_flags(&si->flags))
			return (-1);
		nexttoken();
	}
	else if (t->tokentype == IT_NIL)
		nexttoken();
	else if (t->tokentype == IT_ATOM)
	{
		get_flagname(t->tokenbuf, &si->flags);
		nexttoken();
	}
	return (0);
}

int do_store(unsigned long n, int byuid, void *voidptr)
{
struct storeinfo *si=(struct storeinfo *)voidptr;
int	fd;
struct imapflags new_flags;

	--n;
	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n);
	if (fd < 0)	return (0);

	get_message_flags(current_mailbox_info.msgs+n, 0, &new_flags);
	if (si->plusminus == '+')
	{
		if (si->flags.seen)	new_flags.seen=1;
		if (si->flags.answered)	new_flags.answered=1;
		if (si->flags.deleted)	new_flags.deleted=1;
		if (si->flags.flagged)	new_flags.flagged=1;
	}
	else if (si->plusminus == '-')
	{
		if (si->flags.seen)	new_flags.seen=0;
		if (si->flags.answered)	new_flags.answered=0;
		if (si->flags.deleted)	new_flags.deleted=0;
		if (si->flags.flagged)	new_flags.flagged=0;
	}
	else
		new_flags=si->flags;

	if (reflag_filename(current_mailbox_info.msgs+n, &new_flags, fd))
	{
		close(fd);
		return (-1);
	}
	close(fd);
	if (si->silent)
		current_mailbox_info.msgs[n].changedflags=0;
	else
		fetchflags(n);
	return (0);
}

static int copy_message(int fd, const char *mailbox,
	struct	imapflags *flags)
{
char	*tmpname;
char	*newname;
FILE	*fp;
struct	stat	stat_buf;
char	buf[BUFSIZ];
int	fd2;
int	rc;

	if (fstat(fd, &stat_buf) < 0)
	{
		return (-1);
	}

	maildir_mkfilename(mailbox, flags, stat_buf.st_size, &tmpname, &newname);
	fd2=maildir_safeopen(tmpname, O_CREAT|O_RDWR|O_EXCL, 0644);
	if (fd2 >= 0)
	{
		fp=fdopen(fd2, "w");
		if (fp == 0)
		{
			close(fd2);
			unlink(tmpname);
			fd2= -1;
		}
	}

	if (fd2 < 0)
	{
		free(tmpname);
		free(newname);
		return (-1);
	}

	while (stat_buf.st_size)
	{
	int	n=sizeof(buf);

		if (n > stat_buf.st_size)
			n=stat_buf.st_size;

		n=read(fd, buf, n);

		if (n == 0 || fwrite(buf, 1, n, fp) != n)
		{
			fclose(fp);
			close(fd2);
			unlink(tmpname);
			free(tmpname);
			free(newname);
			return (-1);
		}
		stat_buf.st_size -= n;
	}

	if (fflush(fp) || ferror(fp))
	{
		fclose(fp);
		close(fd2);
		free(tmpname);
		free(newname);
		return (-1);
	}
	fclose(fp);
	close(fd2);

#if	HAVE_UTIME
	{
	struct	utimbuf ub;

		ub.actime=ub.modtime=stat_buf.st_mtime;
		utime(tmpname, &ub);
	}
#else
#if	HAVE_UTIMES
	{
	struct	timeval	tv;

		tv.tv_sec=stat_buf.st_mtime;
		tv.tv_usec=0;
		utimes(tmpname, &tv);
	}
#endif
#endif

	if ((rc=rename(tmpname, newname)) != 0)
		unlink(tmpname);

	free(tmpname);
	free(newname);
	return (rc);
}

int do_copy(unsigned long n, int byuid, void *voidptr)
{
char *mailbox=(char *)voidptr;
int	fd;
struct imapflags new_flags;

	--n;
	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n);
	if (fd < 0)	return (0);
	get_message_flags(current_mailbox_info.msgs+n, 0, &new_flags);

	if (copy_message(fd, mailbox, &new_flags))
	{
		close(fd);
		return (-1);
	}
	close(fd);
	return (0);
}

int do_copy_quota_calc(unsigned long n, int byuid, void *voidptr)
{
struct copyquotainfo *info=(struct copyquotainfo *)voidptr;
const char *filename;
unsigned long nbytes;
struct	stat	stat_buf;
int	fd;

	--n;
	if (is_trash(info->destmailbox))
		return (0);	/* No problems dumping everything into trash */

	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n);
	if (fd < 0)	return (0);
	filename=current_mailbox_info.msgs[n].filename;

	if (maildir_parsequota(filename, &nbytes))
	{
		if (fstat(fd, &stat_buf) < 0)
		{
			close(fd);
			return (0);
		}
		n=stat_buf.st_size;
	}
	close(fd);

	info->nbytes += nbytes;
	info->nfiles += 1;
	return (0);
}

==> courier-imap-0.35.tar.gz: courier-imap-0.35/imap/storeinfo.c <==
/*
** Copyright 1998 - 1999 Double Precision, Inc.
** See COPYING for distribution information.
*/

#if	HAVE_CONFIG_H
#include	"config.h"
#endif
#if     HAVE_UNISTD_H
#include        <unistd.h>
#endif
#include	<errno.h>
#include	"imaptoken.h"
#include	"imapscanclient.h"
#include	"imapwrite.h"
#include	"storeinfo.h"
#include	"maildir/maildirquota.h"
#include	"maildir/maildirmisc.h"

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	<fcntl.h>
#include	<sys/stat.h>
#if	HAVE_UTIME_H
#include	<utime.h>
#endif
#if TIME_WITH_SYS_TIME
#include	<sys/time.h>
#include	<time.h>
#else
#if HAVE_SYS_TIME_H
#include	<sys/time.h>
#else
#include	<time.h>
#endif
#endif

static const char rcsid[]="$Id: storeinfo.c,v 1.6 2000/05/28 00:11:12 mrsam Exp $";

extern int is_trash(const char *);
extern int get_flagname(const char *, struct imapflags *);
extern int get_flags(struct imapflags *);
extern void get_message_flags( struct imapscanmessageinfo *,
	char *, struct imapflags *);
extern int reflag_filename(struct imapscanmessageinfo *, struct imapflags *,
	int);
extern void fetchflags(unsigned long);
extern void maildir_mkfilename(const char *, struct imapflags *,
	unsigned long, char **, char **);

extern struct imapscaninfo current_mailbox_info;
extern char *current_mailbox;

int storeinfo_init(struct storeinfo *si)
{
struct imaptoken *t=currenttoken();
const char *p;

	if (t->tokentype != IT_ATOM)	return (-1);
	si->plusminus=0;
	si->silent=0;

	p=t->tokenbuf;
	if (*p == '+' || *p == '-')
		si->plusminus= *p++;
	if (strncmp(p, "FLAGS", 5))	return (-1);
	p += 5;
	if (*p)
	{
		if (strcmp(p, ".SILENT"))	return (-1);
		si->silent=1;
	}

	memset(&si->flags, 0, sizeof(si->flags));

	t=nexttoken();
	if (t->tokentype == IT_LPAREN)
	{
		if (get_flags(&si->flags))
			return (-1);
		nexttoken();
	}
	else if (t->tokentype == IT_NIL)
		nexttoken();
	else if (t->tokentype == IT_ATOM)
	{
		get_flagname(t->tokenbuf, &si->flags);
		nexttoken();
	}
	return (0);
}

int do_store(unsigned long n, int byuid, void *voidptr)
{
struct storeinfo *si=(struct storeinfo *)voidptr;
int	fd;
struct imapflags new_flags;

	--n;
	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n);
	if (fd < 0)	return (0);

	get_message_flags(current_mailbox_info.msgs+n, 0, &new_flags);
	if (si->plusminus == '+')
	{
		if (si->flags.seen)	new_flags.seen=1;
		if (si->flags.answered)	new_flags.answered=1;
		if (si->flags.deleted)	new_flags.deleted=1;
		if (si->flags.flagged)	new_flags.flagged=1;
	}
	else if (si->plusminus == '-')
	{
		if (si->flags.seen)	new_flags.seen=0;
		if (si->flags.answered)	new_flags.answered=0;
		if (si->flags.deleted)	new_flags.deleted=0;
		if (si->flags.flagged)	new_flags.flagged=0;
	}
	else
		new_flags=si->flags;

	if (reflag_filename(current_mailbox_info.msgs+n, &new_flags, fd))
	{
		close(fd);
		return (-1);
	}
	close(fd);
	if (si->silent)
		current_mailbox_info.msgs[n].changedflags=0;
	else
		fetchflags(n);
	return (0);
}

static int copy_message(int fd, const char *mailbox,
	struct	imapflags *flags)
{
char	*tmpname;
char	*newname;
FILE	*fp;
struct	stat	stat_buf;
char	buf[BUFSIZ];
int	fd2;
int	rc;

	if (fstat(fd, &stat_buf) < 0)
	{
		return (-1);
	}

	maildir_mkfilename(mailbox, flags, stat_buf.st_size, &tmpname, &newname);
	fd2=maildir_safeopen(tmpname, O_CREAT|O_RDWR|O_EXCL, 0644);
	fp=0;
	if (fd2 >= 0)
	{
		fp=fdopen(fd2, "w");
		if (fp == 0)
		{
			close(fd2);
			unlink(tmpname);
			fd2= -1;
		}
	}

	if (fd2 < 0)
	{
		free(tmpname);
		free(newname);
		return (-1);
	}

	while (stat_buf.st_size)
	{
	int	n=sizeof(buf);

		if (n > stat_buf.st_size)
			n=stat_buf.st_size;

		n=read(fd, buf, n);

		if (n <= 0 || fwrite(buf, 1, n, fp) != n)
		{
			fprintf(stderr,
			"ERR: error copying a message, user=%s, errno=%d\n",
				getenv("AUTHENTICATED"), errno);

			fclose(fp);
			close(fd2);
			unlink(tmpname);
			free(tmpname);
			free(newname);
			return (-1);
		}
		stat_buf.st_size -= n;
	}

	if (fflush(fp) || ferror(fp))
	{
		fclose(fp);
		close(fd2);
		free(tmpname);
		free(newname);
		return (-1);
	}
	fclose(fp);
	close(fd2);

#if	HAVE_UTIME
	{
	struct	utimbuf ub;

		ub.actime=ub.modtime=stat_buf.st_mtime;
		utime(tmpname, &ub);
	}
#else
#if	HAVE_UTIMES
	{
	struct	timeval	tv;

		tv.tv_sec=stat_buf.st_mtime;
		tv.tv_usec=0;
		utimes(tmpname, &tv);
	}
#endif
#endif

	if ((rc=rename(tmpname, newname)) != 0)
		unlink(tmpname);

	free(tmpname);
	free(newname);
	return (rc);
}

int do_copy(unsigned long n, int byuid, void *voidptr)
{
char *mailbox=(char *)voidptr;
int	fd;
struct imapflags new_flags;

	--n;
	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n);
	if (fd < 0)	return (0);
	get_message_flags(current_mailbox_info.msgs+n, 0, &new_flags);

	if (copy_message(fd, mailbox, &new_flags))
	{
		close(fd);
		return (-1);
	}
	close(fd);
	return (0);
}

int do_copy_quota_calc(unsigned long n, int byuid, void *voidptr)
{
struct copyquotainfo *info=(struct copyquotainfo *)voidptr;
const char *filename;
unsigned long nbytes;
struct	stat	stat_buf;
int	fd;

	--n;
	if (is_trash(info->destmailbox))
		return (0);	/* No problems dumping everything into trash */

	fd=imapscan_openfile(current_mailbox, &current_mailbox_info, n);
	if (fd < 0)	return (0);
	filename=current_mailbox_info.msgs[n].filename;

	if (maildir_parsequota(filename, &nbytes))
	{
		if (fstat(fd, &stat_buf) < 0)
		{
			close(fd);
			return (0);
		}
		n=stat_buf.st_size;
	}
	close(fd);

	info->nbytes += nbytes;
	info->nfiles += 1;
	return (0);
}
